@@ -218,12 +218,14 @@ remmina_plugin_nx_start_session (RemminaProtocolWidget *gp)
     RemminaPluginNxData *gpdata;
     RemminaFile *remminafile;
     RemminaNXSession *nx;
-    gchar *type, *app;
+    const gchar *type, *app;
     gchar *s1, *s2;
     gint port;
     gint ret;
     gboolean is_empty_list;
     gint event_type = 0;
+    const gchar *cs;
+    gint i;
 
     gpdata = (RemminaPluginNxData*) g_object_get_data (G_OBJECT (gp), "plugin-data");
     remminafile = remmina_plugin_nx_service->protocol_plugin_get_file (gp);
@@ -231,13 +233,21 @@ remmina_plugin_nx_start_session (RemminaProtocolWidget *gp)
 
     /* Connect */
 
-    remmina_nx_session_set_encryption (nx, remminafile->disableencryption ? 0 : 1);
+    remmina_nx_session_set_encryption (nx,
+        remmina_plugin_nx_service->file_get_int (remminafile, "disableencryption", FALSE) ? 0 : 1);
     remmina_nx_session_set_localport (nx, remmina_plugin_nx_service->pref_get_sshtunnel_port ());
     remmina_nx_session_set_log_callback (nx, remmina_plugin_nx_service->log_printf);
 
-    remmina_plugin_nx_service->get_server_port (remminafile->server, 22, &s1, &port);
+    s2 = remmina_plugin_nx_service->protocol_plugin_start_direct_tunnel (gp, 22, FALSE);
+    if (s2 == NULL)
+    {
+        return FALSE;
+    }
+    remmina_plugin_nx_service->get_server_port (s2, 22, &s1, &port);
+    g_free (s2);
 
-    if (!remmina_nx_session_open (nx, s1, port, remminafile->ssh_privatekey,
+    if (!remmina_nx_session_open (nx, s1, port,
+        remmina_plugin_nx_service->file_get_string (remminafile, "nx_privatekey"),
         remmina_plugin_nx_ssh_auth_callback, gp))
     {
         g_free (s1);
@@ -247,11 +257,11 @@ remmina_plugin_nx_start_session (RemminaProtocolWidget *gp)
 
     /* Login */
 
-    if (remminafile->username && remminafile->username[0] &&
-        remminafile->password && remminafile->password[0])
+    if (remmina_plugin_nx_service->file_get_string (remminafile, "username") &&
+        remmina_plugin_nx_service->file_get_string (remminafile, "password"))
     {
-        s1 = g_strdup (remminafile->username);
-        s2 = g_strdup (remminafile->password);
+        s1 = g_strdup (remmina_plugin_nx_service->file_get_string (remminafile, "username"));
+        s2 = g_strdup (remmina_plugin_nx_service->file_get_string (remminafile, "password"));
     }
     else
     {
@@ -273,23 +283,24 @@ remmina_plugin_nx_start_session (RemminaProtocolWidget *gp)
     remmina_plugin_nx_service->protocol_plugin_init_save_cred (gp);
 
     /* Prepare the session type and application */
-    if (!remminafile->exec || !remminafile->exec[0] || g_strcmp0 (remminafile->exec, "GNOME") == 0)
+    cs = remmina_plugin_nx_service->file_get_string (remminafile, "exec");
+    if (!cs || g_strcmp0 (cs, "GNOME") == 0)
     {
         type = "unix-gnome";
         app = NULL;
     }
-    else if (g_strcmp0 (remminafile->exec, "KDE") == 0)
+    else if (g_strcmp0 (cs, "KDE") == 0)
     {
         type = "unix-kde";
         app = NULL;
     }
-    else if (g_strcmp0 (remminafile->exec, "Xfce") == 0)
+    else if (g_strcmp0 (cs, "Xfce") == 0)
     {
         /* NX does not know Xfce. So we simply launch the Xfce startup program. */
         type = "unix-application";
         app = "startxfce4";
     }
-    else if (g_strcmp0 (remminafile->exec, "Shadow") == 0)
+    else if (g_strcmp0 (cs, "Shadow") == 0)
     {
         type = "shadow";
         app = NULL;
@@ -297,7 +308,7 @@ remmina_plugin_nx_start_session (RemminaProtocolWidget *gp)
     else
     {
         type = "unix-application";
-        app = remminafile->exec;
+        app = cs;
     }
 
     /* List sessions */
@@ -308,7 +319,7 @@ remmina_plugin_nx_start_session (RemminaProtocolWidget *gp)
         remmina_nx_session_add_parameter (nx, "type", type);
         if (!gpdata->attach_session)
         {
-            remmina_nx_session_add_parameter (nx, "user", remminafile->username);
+            remmina_nx_session_add_parameter (nx, "user", remmina_plugin_nx_service->file_get_string (remminafile, "username"));
             remmina_nx_session_add_parameter (nx, "status", "suspended,running");
         }
 
@@ -352,16 +363,19 @@ remmina_plugin_nx_start_session (RemminaProtocolWidget *gp)
 
     /* Start, Restore or Attach, based on the setting and existing session */
     remmina_nx_session_add_parameter (nx, "type", type);
+    i = remmina_plugin_nx_service->file_get_int (remminafile, "quality", 0);
     remmina_nx_session_add_parameter (nx, "link",
-        remminafile->quality > 2 ? "lan" :
-        remminafile->quality == 2 ? "adsl" :
-        remminafile->quality == 1 ? "isdn" : "modem");
+        i > 2 ? "lan" :
+        i == 2 ? "adsl" :
+        i == 1 ? "isdn" : "modem");
     remmina_nx_session_add_parameter (nx, "geometry", "%ix%i",
-        remminafile->resolution_width, remminafile->resolution_height);
+        remmina_plugin_nx_service->file_get_int (remminafile, "resolution_width", 0),
+        remmina_plugin_nx_service->file_get_int (remminafile, "resolution_height", 0));
     remmina_nx_session_add_parameter (nx, "keyboard", remmina_kbtype);
     remmina_nx_session_add_parameter (nx, "client", "linux");
     remmina_nx_session_add_parameter (nx, "media", "0");
-    remmina_nx_session_add_parameter (nx, "clipboard", remminafile->disableclipboard ? "none" : "both");
+    remmina_nx_session_add_parameter (nx, "clipboard",
+        remmina_plugin_nx_service->file_get_int (remminafile, "disableclipboard", FALSE) ? "none" : "both");
 
     switch (event_type)
     {
@@ -369,9 +383,10 @@ remmina_plugin_nx_start_session (RemminaProtocolWidget *gp)
     case REMMINA_NX_EVENT_START:
         if (app) remmina_nx_session_add_parameter (nx, "application", app);
 
-        remmina_nx_session_add_parameter (nx, "session", remminafile->name);
+        remmina_nx_session_add_parameter (nx, "session", remmina_plugin_nx_service->file_get_string (remminafile, "name"));
         remmina_nx_session_add_parameter (nx, "screeninfo", "%ix%ix24+render",
-            remminafile->resolution_width, remminafile->resolution_height);
+            remmina_plugin_nx_service->file_get_int (remminafile, "resolution_width", 0),
+            remmina_plugin_nx_service->file_get_int (remminafile, "resolution_height", 0));
 
         if (!remmina_nx_session_start (nx)) return FALSE;
         break;
@@ -393,7 +408,7 @@ remmina_plugin_nx_start_session (RemminaProtocolWidget *gp)
         remmina_nx_session_add_parameter (nx, "id", s1);
         g_free (s1);
 
-        remmina_nx_session_add_parameter (nx, "session", remminafile->name);
+        remmina_nx_session_add_parameter (nx, "session", remmina_plugin_nx_service->file_get_string (remminafile, "name"));
 
         if (!remmina_nx_session_restore (nx)) return FALSE;
         break;
@@ -493,20 +508,25 @@ remmina_plugin_nx_open_connection (RemminaProtocolWidget *gp)
 {
     RemminaPluginNxData *gpdata;
     RemminaFile *remminafile;
+    const gchar *resolution;
+    gint width, height;
 
     gpdata = (RemminaPluginNxData*) g_object_get_data (G_OBJECT (gp), "plugin-data");
     remminafile = remmina_plugin_nx_service->protocol_plugin_get_file (gp);
 
-    if (g_strcmp0 (remminafile->resolution, "AUTO") == 0)
+    resolution = remmina_plugin_nx_service->file_get_string (remminafile, "resolution");
+    if (!resolution || !strchr (resolution, 'x'))
     {
         remmina_plugin_nx_service->protocol_plugin_set_expand (gp, TRUE);
-        gtk_widget_set_size_request (GTK_WIDGET (gp), 320, 240);
+        gtk_widget_set_size_request (GTK_WIDGET (gp), 640, 480);
     }
     else
     {
-        remmina_plugin_nx_service->protocol_plugin_set_width (gp, remminafile->resolution_width);
-        remmina_plugin_nx_service->protocol_plugin_set_height (gp, remminafile->resolution_height);
-        gtk_widget_set_size_request (GTK_WIDGET (gp), remminafile->resolution_width, remminafile->resolution_height);
+        width = remmina_plugin_nx_service->file_get_int (remminafile, "resolution_width", 640);
+        height = remmina_plugin_nx_service->file_get_int (remminafile, "resolution_height", 480);
+        remmina_plugin_nx_service->protocol_plugin_set_width (gp, width);
+        remmina_plugin_nx_service->protocol_plugin_set_height (gp, height);
+        gtk_widget_set_size_request (GTK_WIDGET (gp), width, height);
     }
     gpdata->socket_id = gtk_socket_get_id (GTK_SOCKET (gpdata->socket));
 
@@ -566,49 +586,60 @@ remmina_plugin_nx_close_connection (RemminaProtocolWidget *gp)
     return FALSE;
 }
 
-static gpointer
-remmina_plugin_nx_query_feature (RemminaProtocolWidget *gp, RemminaProtocolFeature feature)
+static gboolean
+remmina_plugin_nx_query_feature (RemminaProtocolWidget *gp, const RemminaProtocolFeature *feature)
 {
-    return NULL;
+    return FALSE;
 }
 
 static void
-remmina_plugin_nx_call_feature (RemminaProtocolWidget *gp, RemminaProtocolFeature feature, const gpointer data)
+remmina_plugin_nx_call_feature (RemminaProtocolWidget *gp, const RemminaProtocolFeature *feature)
 {
 }
 
+static gpointer quality_list[] =
+{
+    "0", N_("Poor (fastest)"),
+    "1", N_("Medium"),
+    "2", N_("Good"),
+    "9", N_("Best (slowest)"),
+    NULL
+};
+
 static const RemminaProtocolSetting remmina_plugin_nx_basic_settings[] =
 {
-    REMMINA_PROTOCOL_SETTING_SERVER,
-    REMMINA_PROTOCOL_SETTING_SSH_PRIVATEKEY,
-    REMMINA_PROTOCOL_SETTING_USERNAME,
-    REMMINA_PROTOCOL_SETTING_PASSWORD,
-    REMMINA_PROTOCOL_SETTING_RESOLUTION_FLEXIBLE,
-    REMMINA_PROTOCOL_SETTING_QUALITY,
-    REMMINA_PROTOCOL_SETTING_EXEC_CUSTOM, "GNOME,KDE,Xfce,Shadow",
-    REMMINA_PROTOCOL_SETTING_CTL_END
+    { REMMINA_PROTOCOL_SETTING_TYPE_SERVER, NULL, NULL, FALSE, NULL, NULL },
+    { REMMINA_PROTOCOL_SETTING_TYPE_FILE, "nx_privatekey", N_("Identity file"), FALSE, NULL, NULL },
+    { REMMINA_PROTOCOL_SETTING_TYPE_TEXT, "username", N_("User name"), FALSE, NULL, NULL },
+    { REMMINA_PROTOCOL_SETTING_TYPE_PASSWORD, NULL, NULL, FALSE, NULL, NULL },
+    { REMMINA_PROTOCOL_SETTING_TYPE_RESOLUTION, NULL, NULL, FALSE, GINT_TO_POINTER (1), NULL },
+    { REMMINA_PROTOCOL_SETTING_TYPE_SELECT, "quality", N_("Quality"), FALSE, quality_list, NULL },
+    { REMMINA_PROTOCOL_SETTING_TYPE_COMBO, "exec", N_("Startup program"), FALSE, "GNOME,KDE,Xfce,Shadow", NULL },
+    { REMMINA_PROTOCOL_SETTING_TYPE_END, NULL, NULL, FALSE, NULL, NULL }
 };
 
 static const RemminaProtocolSetting remmina_plugin_nx_advanced_settings[] =
 {
-    REMMINA_PROTOCOL_SETTING_DISABLECLIPBOARD,
-    REMMINA_PROTOCOL_SETTING_DISABLEENCRYPTION,
-    REMMINA_PROTOCOL_SETTING_SHOWCURSOR_LOCAL,
-    REMMINA_PROTOCOL_SETTING_CTL_END
+    { REMMINA_PROTOCOL_SETTING_TYPE_CHECK, "disableclipboard", N_("Disable clipboard sync"), FALSE, NULL, NULL },
+    { REMMINA_PROTOCOL_SETTING_TYPE_CHECK, "disableencryption", N_("Disable encryption"), FALSE, NULL, NULL },
+    { REMMINA_PROTOCOL_SETTING_TYPE_CHECK, "showcursor", N_("Use local cursor"), FALSE, NULL, NULL },
+    { REMMINA_PROTOCOL_SETTING_TYPE_END, NULL, NULL, FALSE, NULL, NULL }
 };
 
 static RemminaProtocolPlugin remmina_plugin_nx =
 {
     REMMINA_PLUGIN_TYPE_PROTOCOL,
     "NX",
-    NULL,
+    N_("NX - NX Technology"),
+    GETTEXT_PACKAGE,
+    VERSION,
 
     "remmina-nx",
     "remmina-nx",
-    NULL,
     remmina_plugin_nx_basic_settings,
     remmina_plugin_nx_advanced_settings,
-    REMMINA_PROTOCOL_SSH_SETTING_NONE,
+    REMMINA_PROTOCOL_SSH_SETTING_TUNNEL,
+    NULL,
 
     remmina_plugin_nx_init,
     remmina_plugin_nx_open_connection,
@@ -645,7 +676,6 @@ remmina_plugin_entry (RemminaPluginService *service)
         XCloseDisplay (dpy);
     }
 
-    remmina_plugin_nx.description = _("NX - NX Technology");
     if (! service->register_plugin ((RemminaPlugin *) &remmina_plugin_nx))
     {
         return FALSE;
