@@ -24,6 +24,10 @@
 #include "remminapluginrdpfile.h"
 #include "remminapluginrdpset.h"
 
+#define REMMINA_PLUGIN_RDP_FEATURE_TOOL_REFRESH            1
+#define REMMINA_PLUGIN_RDP_FEATURE_SCALE                   2
+#define REMMINA_PLUGIN_RDP_FEATURE_UNFOCUS                 3
+
 RemminaPluginService *remmina_plugin_service = NULL;
 
 static void
@@ -125,6 +129,8 @@ remmina_plugin_rdp_main (RemminaProtocolWidget *gp)
     gchar *s;
     gchar *value;
     gint rdpdr_num;
+    gint drdynvc_num;
+    const gchar *cs;
 
     gpdata = GET_DATA (gp);
     remminafile = remmina_plugin_service->protocol_plugin_get_file (gp);
@@ -139,70 +145,76 @@ remmina_plugin_rdp_main (RemminaProtocolWidget *gp)
     g_free (host);
     g_free (s);
 
-    gpdata->settings->server_depth = remminafile->colordepth;
-    gpdata->settings->width = remminafile->resolution_width;
-    gpdata->settings->height = remminafile->resolution_height;
-    remmina_plugin_service->protocol_plugin_set_width (gp, remminafile->resolution_width);
-    remmina_plugin_service->protocol_plugin_set_height (gp, remminafile->resolution_height);
+    gpdata->settings->server_depth = remmina_plugin_service->file_get_int (remminafile, "colordepth", 8);
+    gpdata->settings->width = remmina_plugin_service->file_get_int (remminafile, "resolution_width", 640);
+    gpdata->settings->height = remmina_plugin_service->file_get_int (remminafile, "resolution_height", 480);
+    remmina_plugin_service->protocol_plugin_set_width (gp, gpdata->settings->width);
+    remmina_plugin_service->protocol_plugin_set_height (gp, gpdata->settings->height);
 
-    if (remminafile->username && remminafile->username[0] != '\0')
+    if (remmina_plugin_service->file_get_string (remminafile, "username"))
     {
-        strncpy (gpdata->settings->username, remminafile->username, sizeof (gpdata->settings->username) - 1);
+        strncpy (gpdata->settings->username, remmina_plugin_service->file_get_string (remminafile, "username"),
+            sizeof (gpdata->settings->username) - 1);
     }
 
-    if (remminafile->domain && remminafile->domain[0] != '\0')
+    if (remmina_plugin_service->file_get_string (remminafile, "domain"))
     {
-        strncpy (gpdata->settings->domain, remminafile->domain, sizeof (gpdata->settings->domain) - 1);
+        strncpy (gpdata->settings->domain, remmina_plugin_service->file_get_string (remminafile, "domain"),
+            sizeof (gpdata->settings->domain) - 1);
     }
 
-    if (remminafile->password && remminafile->password[0] != '\0')
+    if (remmina_plugin_service->file_get_string (remminafile, "password"))
     {
-        strncpy (gpdata->settings->password, remminafile->password, sizeof (gpdata->settings->password) - 1);
+        strncpy (gpdata->settings->password, remmina_plugin_service->file_get_string (remminafile, "password"),
+            sizeof (gpdata->settings->password) - 1);
         gpdata->settings->autologin = 1;
     }
 
-    if (remminafile->clientname && remminafile->clientname[0] != '\0')
+    if (remmina_plugin_service->file_get_string (remminafile, "clientname"))
     {
-        strncpy (gpdata->settings->hostname, remminafile->clientname, sizeof (gpdata->settings->hostname) - 1);
+        strncpy (gpdata->settings->hostname, remmina_plugin_service->file_get_string (remminafile, "clientname"),
+            sizeof (gpdata->settings->hostname) - 1);
     }
     else
     {
         strncpy (gpdata->settings->hostname, g_get_host_name (), sizeof (gpdata->settings->hostname) - 1);
     }
 
-    if (remminafile->exec && remminafile->exec[0] != '\0')
+    if (remmina_plugin_service->file_get_string (remminafile, "exec"))
     {
-        strncpy (gpdata->settings->shell, remminafile->exec, sizeof (gpdata->settings->shell) - 1);
+        strncpy (gpdata->settings->shell, remmina_plugin_service->file_get_string (remminafile, "exec"),
+            sizeof (gpdata->settings->shell) - 1);
     }
 
-    if (remminafile->execpath && remminafile->execpath[0] != '\0')
+    if (remmina_plugin_service->file_get_string (remminafile, "execpath"))
     {
-        strncpy (gpdata->settings->directory, remminafile->execpath, sizeof (gpdata->settings->directory) - 1);
+        strncpy (gpdata->settings->directory, remmina_plugin_service->file_get_string (remminafile, "execpath"),
+            sizeof (gpdata->settings->directory) - 1);
     }
 
-    s = g_strdup_printf ("rdp_quality_%i", remminafile->quality);
+    s = g_strdup_printf ("rdp_quality_%i", remmina_plugin_service->file_get_int (remminafile, "quality", DEFAULT_QUALITY_0));
     value = remmina_plugin_service->pref_get_value (s);
     g_free (s);
     if (value && value[0])
     {
-        gpdata->settings->rdp5_performanceflags = strtoul (value, NULL, 16);
+        gpdata->settings->performanceflags = strtoul (value, NULL, 16);
     }
     else
     {
-        switch (remminafile->quality)
+        switch (remmina_plugin_service->file_get_int (remminafile, "quality", DEFAULT_QUALITY_0))
         {
         case 9:
-            gpdata->settings->rdp5_performanceflags = DEFAULT_QUALITY_9;
+            gpdata->settings->performanceflags = DEFAULT_QUALITY_9;
             break;
         case 2:
-            gpdata->settings->rdp5_performanceflags = DEFAULT_QUALITY_2;
+            gpdata->settings->performanceflags = DEFAULT_QUALITY_2;
             break;
         case 1:
-            gpdata->settings->rdp5_performanceflags = DEFAULT_QUALITY_1;
+            gpdata->settings->performanceflags = DEFAULT_QUALITY_1;
             break;
         case 0:
         default:
-            gpdata->settings->rdp5_performanceflags = DEFAULT_QUALITY_0;
+            gpdata->settings->performanceflags = DEFAULT_QUALITY_0;
             break;
         }
     }
@@ -210,37 +222,51 @@ remmina_plugin_rdp_main (RemminaProtocolWidget *gp)
 
     gpdata->settings->keyboard_layout = remmina_plugin_rdpset_get_keyboard_layout();
 
-    if (g_strcmp0 (remminafile->sound, "remote") == 0)
-    {
-        gpdata->settings->console_audio = 1;
-    }
-    else if (g_strcmp0 (remminafile->sound, "local") == 0)
-    {
-        freerdp_chanman_load_plugin (gpdata->chan_man, gpdata->settings, "rdpsnd", NULL);
-    }
-
-    if (remminafile->console)
+    if (remmina_plugin_service->file_get_int (remminafile, "console", FALSE))
     {
         gpdata->settings->console_session = 1;
     }
 
-    if (!remminafile->disableclipboard)
+    if (remmina_plugin_service->file_get_int (remminafile, "disabletls", FALSE))
+    {
+        gpdata->settings->tls = 0;
+    }
+
+    drdynvc_num = 0;
+    if (g_strcmp0 (remmina_plugin_service->file_get_string (remminafile, "sound"), "remote") == 0)
+    {
+        gpdata->settings->console_audio = 1;
+    }
+    else if (g_strcmp0 (remmina_plugin_service->file_get_string (remminafile, "sound"), "local") == 0)
+    {
+        freerdp_chanman_load_plugin (gpdata->chan_man, gpdata->settings, "rdpsnd", NULL);
+        gpdata->drdynvc_data[drdynvc_num].size = sizeof(RD_PLUGIN_DATA);
+        gpdata->drdynvc_data[drdynvc_num].data[0] = "audin";
+        drdynvc_num++;
+    }
+    if (drdynvc_num)
+    {
+        freerdp_chanman_load_plugin (gpdata->chan_man, gpdata->settings, "drdynvc", gpdata->drdynvc_data);
+    }
+
+    if (!remmina_plugin_service->file_get_int (remminafile, "disableclipboard", FALSE))
     {
         freerdp_chanman_load_plugin (gpdata->chan_man, gpdata->settings, "cliprdr", NULL);
     }
 
     rdpdr_num = 0;
-    if (remminafile->sharefolder && remminafile->sharefolder[0] == '/')
+    cs = remmina_plugin_service->file_get_string (remminafile, "sharefolder");
+    if (cs && cs[0] == '/')
     {
-        s = strrchr (remminafile->sharefolder, '/');
+        s = strrchr (cs, '/');
         s = (s && s[1] ? s + 1 : "root");
         gpdata->rdpdr_data[rdpdr_num].size = sizeof(RD_PLUGIN_DATA);
         gpdata->rdpdr_data[rdpdr_num].data[0] = "disk";
         gpdata->rdpdr_data[rdpdr_num].data[1] = s;
-        gpdata->rdpdr_data[rdpdr_num].data[2] = remminafile->sharefolder;
+        gpdata->rdpdr_data[rdpdr_num].data[2] = (gchar*) cs;
         rdpdr_num++;
     }
-    if (remminafile->shareprinter)
+    if (remmina_plugin_service->file_get_int (remminafile, "shareprinter", FALSE))
     {
         gpdata->rdpdr_data[rdpdr_num].size = sizeof(RD_PLUGIN_DATA);
         gpdata->rdpdr_data[rdpdr_num].data[0] = "printer";
@@ -328,11 +354,12 @@ remmina_plugin_rdp_init (RemminaProtocolWidget *gp)
     gpdata->settings->bitmap_cache = 1;
     gpdata->settings->bitmap_compression = 1;
     gpdata->settings->desktop_save = 0;
-    gpdata->settings->rdp5_performanceflags = 0;
+    gpdata->settings->performanceflags = 0;
     gpdata->settings->off_screen_bitmaps = 1;
     gpdata->settings->triblt = 0;
     gpdata->settings->new_cursors = 1;
     gpdata->settings->rdp_version = 5;
+    gpdata->settings->tls = 1;
 
     pthread_mutex_init (&gpdata->mutex, NULL);
 
@@ -374,6 +401,10 @@ remmina_plugin_rdp_close_connection (RemminaProtocolWidget *gp)
     remmina_plugin_rdpev_uninit (gp);
     remmina_plugin_rdpui_uninit (gp);
 
+    if (gpdata->chan_man && gpdata->inst)
+    {
+        freerdp_chanman_close (gpdata->chan_man, gpdata->inst);
+    }
     if (gpdata->inst)
     {
         gpdata->inst->rdp_disconnect (gpdata->inst);
@@ -396,36 +427,30 @@ remmina_plugin_rdp_close_connection (RemminaProtocolWidget *gp)
     return FALSE;
 }
 
-static gpointer
-remmina_plugin_rdp_query_feature (RemminaProtocolWidget *gp, RemminaProtocolFeature feature)
+static gboolean
+remmina_plugin_rdp_query_feature (RemminaProtocolWidget *gp, const RemminaProtocolFeature *feature)
 {
-    switch (feature)
-    {
-        case REMMINA_PROTOCOL_FEATURE_UNFOCUS:
-        case REMMINA_PROTOCOL_FEATURE_SCALE:
-        case REMMINA_PROTOCOL_FEATURE_TOOL_REFRESH:
-            return GINT_TO_POINTER (1);
-        default:
-            return NULL;
-    }
+    return TRUE;
 }
 
 static void
-remmina_plugin_rdp_call_feature (RemminaProtocolWidget *gp, RemminaProtocolFeature feature, const gpointer data)
+remmina_plugin_rdp_call_feature (RemminaProtocolWidget *gp, const RemminaProtocolFeature *feature)
 {
     RemminaPluginRdpData *gpdata;
+    RemminaFile *remminafile;
 
     gpdata = GET_DATA (gp);
-    switch (feature)
+    remminafile = remmina_plugin_service->protocol_plugin_get_file (gp);
+    switch (feature->id)
     {
-        case REMMINA_PROTOCOL_FEATURE_UNFOCUS:
+        case REMMINA_PLUGIN_RDP_FEATURE_UNFOCUS:
             remmina_plugin_rdpev_unfocus (gp);
             break;
-        case REMMINA_PROTOCOL_FEATURE_SCALE:
-            gpdata->scale = (data != NULL);
+        case REMMINA_PLUGIN_RDP_FEATURE_SCALE:
+            gpdata->scale = remmina_plugin_service->file_get_int (remminafile, "scale", FALSE);
             remmina_plugin_rdpev_update_scale (gp);
             break;
-        case REMMINA_PROTOCOL_FEATURE_TOOL_REFRESH:
+        case REMMINA_PLUGIN_RDP_FEATURE_TOOL_REFRESH:
             LOCK_BUFFER (FALSE)
             remmina_plugin_rdpui_update_rect (gp, 0, 0,
                 remmina_plugin_service->protocol_plugin_get_width (gp),
@@ -437,43 +462,80 @@ remmina_plugin_rdp_call_feature (RemminaProtocolWidget *gp, RemminaProtocolFeatu
     }
 }
 
+static gpointer colordepth_list[] =
+{
+    "8", N_("256 colors"),
+    "15", N_("High color (15 bit)"),
+    "16", N_("High color (16 bit)"),
+    "24", N_("True color (24 bit)"),
+    NULL
+};
+
+static gpointer quality_list[] =
+{
+    "0", N_("Poor (fastest)"),
+    "1", N_("Medium"),
+    "2", N_("Good"),
+    "9", N_("Best (slowest)"),
+    NULL
+};
+
+static gpointer sound_list[] =
+{
+    "off", N_("Off"),
+    "local", N_("Local"),
+    "remote", N_("Remote"),
+    NULL
+};
+
 static const RemminaProtocolSetting remmina_plugin_rdp_basic_settings[] =
 {
-    REMMINA_PROTOCOL_SETTING_SERVER,
-    REMMINA_PROTOCOL_SETTING_USERNAME,
-    REMMINA_PROTOCOL_SETTING_PASSWORD,
-    REMMINA_PROTOCOL_SETTING_DOMAIN,
-    REMMINA_PROTOCOL_SETTING_RESOLUTION_FIXED,
-    REMMINA_PROTOCOL_SETTING_COLORDEPTH,
-    REMMINA_PROTOCOL_SETTING_SHAREFOLDER,
-    REMMINA_PROTOCOL_SETTING_CTL_END
+    { REMMINA_PROTOCOL_SETTING_TYPE_SERVER, NULL, NULL, FALSE, NULL, NULL },
+    { REMMINA_PROTOCOL_SETTING_TYPE_TEXT, "username", N_("User name"), FALSE, NULL, NULL },
+    { REMMINA_PROTOCOL_SETTING_TYPE_PASSWORD, NULL, NULL, FALSE, NULL, NULL },
+    { REMMINA_PROTOCOL_SETTING_TYPE_TEXT, "domain", N_("Domain"), FALSE, NULL, NULL },
+    { REMMINA_PROTOCOL_SETTING_TYPE_RESOLUTION, NULL, NULL, FALSE, NULL, NULL },
+    { REMMINA_PROTOCOL_SETTING_TYPE_SELECT, "colordepth", N_("Color depth"), FALSE, colordepth_list, NULL },
+    { REMMINA_PROTOCOL_SETTING_TYPE_FOLDER, "sharefolder", N_("Share folder"), FALSE, NULL, NULL },
+    { REMMINA_PROTOCOL_SETTING_TYPE_END, NULL, NULL, FALSE, NULL, NULL }
 };
 
 static const RemminaProtocolSetting remmina_plugin_rdp_advanced_settings[] =
 {
-    REMMINA_PROTOCOL_SETTING_QUALITY,
-    REMMINA_PROTOCOL_SETTING_SOUND,
-    REMMINA_PROTOCOL_SETTING_CLIENTNAME,
-    REMMINA_PROTOCOL_SETTING_EXEC,
-    REMMINA_PROTOCOL_SETTING_EXECPATH,
-    REMMINA_PROTOCOL_SETTING_SHAREPRINTER,
-    REMMINA_PROTOCOL_SETTING_DISABLECLIPBOARD,
-    REMMINA_PROTOCOL_SETTING_CONSOLE,
-    REMMINA_PROTOCOL_SETTING_CTL_END
+    { REMMINA_PROTOCOL_SETTING_TYPE_SELECT, "quality", N_("Quality"), FALSE, quality_list, NULL },
+    { REMMINA_PROTOCOL_SETTING_TYPE_SELECT, "sound", N_("Sound"), FALSE, sound_list, NULL },
+    { REMMINA_PROTOCOL_SETTING_TYPE_TEXT, "clientname", N_("Client name"), FALSE, NULL, NULL },
+    { REMMINA_PROTOCOL_SETTING_TYPE_TEXT, "exec", N_("Startup program"), FALSE, NULL, NULL },
+    { REMMINA_PROTOCOL_SETTING_TYPE_TEXT, "execpath", N_("Startup path"), FALSE, NULL, NULL },
+    { REMMINA_PROTOCOL_SETTING_TYPE_CHECK, "shareprinter", N_("Share local printers"), TRUE, NULL, NULL },
+    { REMMINA_PROTOCOL_SETTING_TYPE_CHECK, "disableclipboard", N_("Disable clipboard sync"), FALSE, NULL, NULL },
+    { REMMINA_PROTOCOL_SETTING_TYPE_CHECK, "disabletls", N_("Disable TLS negotiation"), FALSE, NULL, NULL },
+    { REMMINA_PROTOCOL_SETTING_TYPE_CHECK, "console", N_("Attach to console (Windows 2003 / 2003 R2)"), FALSE, NULL, NULL },
+    { REMMINA_PROTOCOL_SETTING_TYPE_END, NULL, NULL, FALSE, NULL, NULL }
+};
+
+static const RemminaProtocolFeature remmina_plugin_rdp_features[] =
+{
+    { REMMINA_PROTOCOL_FEATURE_TYPE_TOOL, REMMINA_PLUGIN_RDP_FEATURE_TOOL_REFRESH, N_("Refresh"), GTK_STOCK_REFRESH, NULL },
+    { REMMINA_PROTOCOL_FEATURE_TYPE_SCALE, REMMINA_PLUGIN_RDP_FEATURE_SCALE, NULL, NULL, NULL },
+    { REMMINA_PROTOCOL_FEATURE_TYPE_UNFOCUS, REMMINA_PLUGIN_RDP_FEATURE_UNFOCUS, NULL, NULL, NULL },
+    { REMMINA_PROTOCOL_FEATURE_TYPE_END, 0, NULL, NULL, NULL }
 };
 
 static RemminaProtocolPlugin remmina_plugin_rdp =
 {
     REMMINA_PLUGIN_TYPE_PROTOCOL,
     "RDP",
-    NULL,
+    N_("RDP - Windows Terminal Service"),
+    GETTEXT_PACKAGE,
+    VERSION,
 
     "remmina-rdp",
     "remmina-rdp-ssh",
-    NULL,
     remmina_plugin_rdp_basic_settings,
     remmina_plugin_rdp_advanced_settings,
     REMMINA_PROTOCOL_SSH_SETTING_TUNNEL,
+    remmina_plugin_rdp_features,
 
     remmina_plugin_rdp_init,
     remmina_plugin_rdp_open_connection,
@@ -486,7 +548,9 @@ static RemminaFilePlugin remmina_plugin_rdpf =
 {
     REMMINA_PLUGIN_TYPE_FILE,
     "RDPF",
-    NULL,
+    N_("RDP - RDP File Handler"),
+    GETTEXT_PACKAGE,
+    VERSION,
 
     remmina_plugin_rdp_file_import_test,
     remmina_plugin_rdp_file_import,
@@ -499,7 +563,9 @@ static RemminaPrefPlugin remmina_plugin_rdps =
 {
     REMMINA_PLUGIN_TYPE_PREF,
     "RDPS",
-    NULL,
+    N_("RDP - Preferences"),
+    GETTEXT_PACKAGE,
+    VERSION,
 
     "RDP",
     remmina_plugin_rdpset_new
@@ -513,18 +579,15 @@ remmina_plugin_entry (RemminaPluginService *service)
     bindtextdomain (GETTEXT_PACKAGE, REMMINA_LOCALEDIR);
     bind_textdomain_codeset (GETTEXT_PACKAGE, "UTF-8");
 
-    remmina_plugin_rdp.description = _("RDP - Windows Terminal Service");
     if (! service->register_plugin ((RemminaPlugin *) &remmina_plugin_rdp))
     {
         return FALSE;
     }
-    remmina_plugin_rdpf.description = _("RDP - RDP File Handler");
     remmina_plugin_rdpf.export_hints = _("Export connection in Windows .rdp file format");
     if (! service->register_plugin ((RemminaPlugin *) &remmina_plugin_rdpf))
     {
         return FALSE;
     }
-    remmina_plugin_rdps.description = _("RDP - Preferences");
     if (! service->register_plugin ((RemminaPlugin *) &remmina_plugin_rdps))
     {
         return FALSE;
