@@ -50,6 +50,7 @@ typedef struct _RemminaPluginRdpData
     gboolean scale;
 
     RD_PLUGIN_DATA rdpdr_data[5];
+    RD_PLUGIN_DATA drdynvc_data[5];
 
     GtkWidget *drawing_area;
     GdkPixbuf *drw_buffer;
@@ -58,7 +59,7 @@ typedef struct _RemminaPluginRdpData
     gint scale_width;
     gint scale_height;
     guint scale_handler;
-    guchar *colourmap;
+    guchar *colormap;
     gint clip_x, clip_y, clip_w, clip_h;
     gboolean clip;
     guchar fgcolor[3];
@@ -67,6 +68,7 @@ typedef struct _RemminaPluginRdpData
     gint pattern_w, pattern_h;
     gboolean capslock_initstate;
     gboolean numlock_initstate;
+    gboolean use_client_keymap;
 
     gint queuedraw_x, queuedraw_y, queuedraw_w, queuedraw_h;
     guint queuedraw_handler;
