@@ -21,7 +21,9 @@
 #include "remminapluginrdp.h"
 #include "remminapluginrdpev.h"
 #include <gdk/gdkkeysyms.h>
+#include <gdk/gdkx.h>
 #include <freerdp/kbd.h>
+#include <X11/Xlib.h>
 
 static void
 remmina_plugin_rdpev_event_push (RemminaProtocolWidget *gp,
@@ -84,6 +86,7 @@ remmina_plugin_rdpev_update_scale_buffer (RemminaProtocolWidget *gp)
     gint width, height;
     gint gpwidth, gpheight;
     gboolean scale;
+    gint hscale, vscale;
 
     gpdata = GET_DATA (gp);
     remminafile = remmina_plugin_service->protocol_plugin_get_file (gp);
@@ -103,10 +106,12 @@ remmina_plugin_rdpev_update_scale_buffer (RemminaProtocolWidget *gp)
             }
             gpwidth = remmina_plugin_service->protocol_plugin_get_width (gp);
             gpheight = remmina_plugin_service->protocol_plugin_get_height (gp);
-            gpdata->scale_width = (remminafile->hscale > 0 ?
-                MAX (1, gpwidth * remminafile->hscale / 100) : width);
-            gpdata->scale_height = (remminafile->vscale > 0 ?
-                MAX (1, gpheight * remminafile->vscale / 100) : height);
+            hscale = remmina_plugin_service->file_get_int (remminafile, "hscale", 0);
+            vscale = remmina_plugin_service->file_get_int (remminafile, "vscale", 0);
+            gpdata->scale_width = (hscale > 0 ?
+                MAX (1, gpwidth * hscale / 100) : width);
+            gpdata->scale_height = (vscale > 0 ?
+                MAX (1, gpheight * vscale / 100) : height);
 
             if (gpdata->scale_width == gpwidth && gpdata->scale_height == gpheight)
             {
@@ -297,6 +302,8 @@ remmina_plugin_rdpev_on_key (GtkWidget *widget, GdkEventKey *event, RemminaProto
     RemminaPluginRdpData *gpdata;
     gint flag;
     gint scancode = 0;
+    Display *display;
+    KeyCode cooked_keycode;
 
     gpdata = GET_DATA (gp);
     flag = (event->type == GDK_KEY_PRESS ? RDP_KEYPRESS : RDP_KEYRELEASE);
@@ -310,9 +317,20 @@ remmina_plugin_rdpev_on_key (GtkWidget *widget, GdkEventKey *event, RemminaProto
         remmina_plugin_rdpev_event_push (gp, RDP_INPUT_SCANCODE, flag, 0x45, 0);
         break;
     default:
-        scancode = freerdp_kbd_get_scancode_by_keycode (event->hardware_keycode, &flag);
-        remmina_plugin_service->log_printf ("[RDP]keyval=%04X keycode=%i scancode=%i flag=%04X\n",
-            event->keyval, event->hardware_keycode, scancode, flag);
+        if (!gpdata->use_client_keymap)
+        {
+            scancode = freerdp_kbd_get_scancode_by_keycode (event->hardware_keycode, &flag);
+            remmina_plugin_service->log_printf ("[RDP]keyval=%04X keycode=%i scancode=%i flag=%04X\n",
+                event->keyval, event->hardware_keycode, scancode, flag);
+        }
+        else
+        {
+            display = GDK_DISPLAY_XDISPLAY (gdk_display_get_default ());
+            cooked_keycode = XKeysymToKeycode(display, XKeycodeToKeysym(display, event->hardware_keycode, 0));
+            scancode = freerdp_kbd_get_scancode_by_keycode (cooked_keycode, &flag);
+            remmina_plugin_service->log_printf ("[RDP]keyval=%04X raw_keycode=%i cooked_keycode=%i scancode=%i flag=%04X\n",
+                event->keyval, event->hardware_keycode, cooked_keycode, scancode, flag);
+        }
         if (scancode)
         {
             remmina_plugin_rdpev_event_push (gp, RDP_INPUT_SCANCODE, flag, scancode, 0);
@@ -340,6 +358,7 @@ remmina_plugin_rdpev_init (RemminaProtocolWidget *gp)
 {
     RemminaPluginRdpData *gpdata;
     gint flags;
+    gchar *s;
 
     gpdata = GET_DATA (gp);
     gpdata->drawing_area = gtk_drawing_area_new ();
@@ -352,6 +371,10 @@ remmina_plugin_rdpev_init (RemminaProtocolWidget *gp)
 
     remmina_plugin_service->protocol_plugin_register_hostkey (gp, gpdata->drawing_area);
 
+    s = remmina_plugin_service->pref_get_value ("rdp_use_client_keymap");
+    gpdata->use_client_keymap = (s && s[0] == '1' ? TRUE : FALSE);
+    g_free (s);
+
     g_signal_connect (G_OBJECT (gpdata->drawing_area), "expose_event",
         G_CALLBACK (remmina_plugin_rdpev_on_expose), gp);
     g_signal_connect (G_OBJECT (gpdata->drawing_area), "configure_event",
@@ -418,17 +441,20 @@ remmina_plugin_rdpev_update_scale (RemminaProtocolWidget *gp)
     RemminaPluginRdpData *gpdata;
     RemminaFile *remminafile;
     gint width, height;
+    gint hscale, vscale;
 
     gpdata = GET_DATA (gp);
     remminafile = remmina_plugin_service->protocol_plugin_get_file (gp);
 
     width = remmina_plugin_service->protocol_plugin_get_width (gp);
     height = remmina_plugin_service->protocol_plugin_get_height (gp);
+    hscale = remmina_plugin_service->file_get_int (remminafile, "hscale", 0);
+    vscale = remmina_plugin_service->file_get_int (remminafile, "vscale", 0);
     if (gpdata->scale)
     {
         gtk_widget_set_size_request (GTK_WIDGET (gpdata->drawing_area),
-            (remminafile->hscale > 0 ? width * remminafile->hscale / 100 : -1),
-            (remminafile->vscale > 0 ? height * remminafile->vscale / 100 : -1));
+            (hscale > 0 ? width * hscale / 100 : -1),
+            (vscale > 0 ? height * vscale / 100 : -1));
     }
     else
     {
