@@ -20,6 +20,14 @@
 
 #include "common/remminaplugincommon.h"
 
+#define REMMINA_PLUGIN_VNC_FEATURE_PREF_QUALITY            1
+#define REMMINA_PLUGIN_VNC_FEATURE_PREF_VIEWONLY           2
+#define REMMINA_PLUGIN_VNC_FEATURE_PREF_DISABLESERVERINPUT 3
+#define REMMINA_PLUGIN_VNC_FEATURE_TOOL_REFRESH            4
+#define REMMINA_PLUGIN_VNC_FEATURE_TOOL_CHAT               5
+#define REMMINA_PLUGIN_VNC_FEATURE_SCALE                   6
+#define REMMINA_PLUGIN_VNC_FEATURE_UNFOCUS                 7
+
 typedef struct _RemminaPluginVncData
 {
     /* Whether the user requests to connect/disconnect */
@@ -231,6 +239,7 @@ remmina_plugin_vnc_update_scale_buffer (RemminaProtocolWidget *gp, gboolean in_t
     RemminaFile *remminafile;
     gint width, height;
     gint gpwidth, gpheight;
+    gint hscale, vscale;
     gboolean scale;
     gint x, y, w, h;
     GdkPixbuf *pixbuf;
@@ -255,10 +264,12 @@ remmina_plugin_vnc_update_scale_buffer (RemminaProtocolWidget *gp, gboolean in_t
                 }
                 gpwidth = remmina_plugin_service->protocol_plugin_get_width (gp);
                 gpheight = remmina_plugin_service->protocol_plugin_get_height (gp);
-                gpdata->scale_width = (remminafile->hscale > 0 ?
-                    MAX (1, gpwidth * remminafile->hscale / 100) : width);
-                gpdata->scale_height = (remminafile->vscale > 0 ?
-                    MAX (1, gpheight * remminafile->vscale / 100) : height);
+                hscale = remmina_plugin_service->file_get_int (remminafile, "hscale", 0);
+                vscale = remmina_plugin_service->file_get_int (remminafile, "vscale", 0);
+                gpdata->scale_width = (hscale > 0 ?
+                    MAX (1, gpwidth * hscale / 100) : width);
+                gpdata->scale_height = (vscale > 0 ?
+                    MAX (1, gpheight * vscale / 100) : height);
 
                 pixbuf = gdk_pixbuf_new (GDK_COLORSPACE_RGB, FALSE, 8,
                     gpdata->scale_width, gpdata->scale_height);
@@ -314,6 +325,7 @@ remmina_plugin_vnc_update_scale (RemminaProtocolWidget *gp, gboolean scale)
     RemminaPluginVncData *gpdata;
     RemminaFile *remminafile;
     gint width, height;
+    gint hscale, vscale;
 
     gpdata = (RemminaPluginVncData*) g_object_get_data (G_OBJECT (gp), "plugin-data");
     remminafile = remmina_plugin_service->protocol_plugin_get_file (gp);
@@ -322,9 +334,11 @@ remmina_plugin_vnc_update_scale (RemminaProtocolWidget *gp, gboolean scale)
     height = remmina_plugin_service->protocol_plugin_get_height (gp);
     if (scale)
     {
+        hscale = remmina_plugin_service->file_get_int (remminafile, "hscale", 0);
+        vscale = remmina_plugin_service->file_get_int (remminafile, "vscale", 0);
         gtk_widget_set_size_request (GTK_WIDGET (gpdata->drawing_area),
-            (remminafile->hscale > 0 ? width * remminafile->hscale / 100 : -1),
-            (remminafile->vscale > 0 ? height * remminafile->vscale / 100 : -1));
+            (hscale > 0 ? width * hscale / 100 : -1),
+            (vscale > 0 ? height * vscale / 100 : -1));
     }
     else
     {
@@ -390,7 +404,9 @@ typedef struct _RemminaKeyVal
 /***************************** LibVNCClient related codes *********************************/
 #include <rfb/rfbclient.h>
 
+#ifdef LIBVNCSERVER_WITH_CLIENT_TLS
 static const uint32_t remmina_plugin_vnc_no_encrypt_auth_types[] = {rfbNoAuth, rfbVncAuth, rfbMSLogon, 0};
+#endif
 
 static void
 remmina_plugin_vnc_process_vnc_event (RemminaProtocolWidget *gp)
@@ -799,9 +815,9 @@ remmina_plugin_vnc_rfb_password (rfbClient *cl)
     gpdata->auth_called = TRUE;
     remminafile = remmina_plugin_service->protocol_plugin_get_file (gp);
 
-    if (gpdata->auth_first && remminafile->password && remminafile->password[0] != '\0')
+    if (gpdata->auth_first && remmina_plugin_service->file_get_string (remminafile, "password"))
     {
-        pwd = g_strdup (remminafile->password);
+        pwd = g_strdup (remmina_plugin_service->file_get_string (remminafile, "password"));
     }
     else
     {
@@ -829,7 +845,6 @@ remmina_plugin_vnc_rfb_credential (rfbClient *cl, int credentialType)
     RemminaProtocolWidget *gp;
     RemminaPluginVncData *gpdata;
     RemminaFile *remminafile;
-    gchar *s;
     gint ret;
 
     gp = (RemminaProtocolWidget*) (rfbClientGetClientData (cl, NULL));
@@ -845,11 +860,11 @@ remmina_plugin_vnc_rfb_credential (rfbClient *cl, int credentialType)
     case rfbCredentialTypeUser:
 
         if (gpdata->auth_first && 
-            remminafile->username && remminafile->username[0] != '\0' &&
-            remminafile->password && remminafile->password[0] != '\0')
+            remmina_plugin_service->file_get_string (remminafile, "username") &&
+            remmina_plugin_service->file_get_string (remminafile, "password"))
         {
-            cred->userCredential.username = g_strdup (remminafile->username);
-            cred->userCredential.password = g_strdup (remminafile->password);
+            cred->userCredential.username = g_strdup (remmina_plugin_service->file_get_string (remminafile, "username"));
+            cred->userCredential.password = g_strdup (remmina_plugin_service->file_get_string (remminafile, "password"));
         }
         else
         {
@@ -873,16 +888,12 @@ remmina_plugin_vnc_rfb_credential (rfbClient *cl, int credentialType)
 
     case rfbCredentialTypeX509:
         if (gpdata->auth_first &&
-            remminafile->cacert && remminafile->cacert[0] != '\0')
+            remmina_plugin_service->file_get_string (remminafile, "cacert"))
         {
-            s = remminafile->cacert;
-            cred->x509Credential.x509CACertFile = (s && s[0] ? g_strdup (s) : NULL);
-            s = remminafile->cacrl;
-            cred->x509Credential.x509CACrlFile = (s && s[0] ? g_strdup (s) : NULL);
-            s = remminafile->clientcert;
-            cred->x509Credential.x509ClientCertFile = (s && s[0] ? g_strdup (s) : NULL);
-            s = remminafile->clientkey;
-            cred->x509Credential.x509ClientKeyFile = (s && s[0] ? g_strdup (s) : NULL);
+            cred->x509Credential.x509CACertFile = g_strdup (remmina_plugin_service->file_get_string (remminafile, "cacert"));
+            cred->x509Credential.x509CACrlFile = g_strdup (remmina_plugin_service->file_get_string (remminafile, "cacrl"));
+            cred->x509Credential.x509ClientCertFile = g_strdup (remmina_plugin_service->file_get_string (remminafile, "clientcert"));
+            cred->x509Credential.x509ClientKeyFile = g_strdup (remmina_plugin_service->file_get_string (remminafile, "clientkey"));
         }
         else
         {
@@ -1154,6 +1165,7 @@ remmina_plugin_vnc_main (RemminaProtocolWidget *gp)
     RemminaFile *remminafile;
     rfbClient *cl = NULL;
     gchar *host;
+    gchar *s = NULL;
 
     remminafile = remmina_plugin_service->protocol_plugin_get_file (gp);
     gpdata = (RemminaPluginVncData*) g_object_get_data (G_OBJECT (gp), "plugin-data");
@@ -1182,7 +1194,8 @@ remmina_plugin_vnc_main (RemminaProtocolWidget *gp)
         cl->GetCredential = remmina_plugin_vnc_rfb_credential;
 #endif
         cl->GotFrameBufferUpdate = remmina_plugin_vnc_rfb_updatefb;
-        cl->GotXCutText = (remminafile->disableclipboard ? NULL : remmina_plugin_vnc_rfb_cuttext);
+        cl->GotXCutText = (remmina_plugin_service->file_get_int (remminafile, "disableclipboard", FALSE) ?
+            NULL : remmina_plugin_vnc_rfb_cuttext);
         cl->GotCursorShape = remmina_plugin_vnc_rfb_cursor_shape;
         cl->Bell = remmina_plugin_vnc_rfb_bell;
         cl->HandleTextChat = remmina_plugin_vnc_rfb_chat;
@@ -1190,9 +1203,9 @@ remmina_plugin_vnc_main (RemminaProtocolWidget *gp)
 
         if (host[0] == '\0')
         {
-            cl->serverHost = host;
+            cl->serverHost = strdup (host);
             cl->listenSpecified = TRUE;
-            if (remminafile->ssh_enabled)
+            if (remmina_plugin_service->file_get_int (remminafile, "ssh_enabled", FALSE))
             {
                 /* When we use reverse tunnel, the local port does not really matter.
                  * Hardcode a default port just in case the remote port is customized
@@ -1201,34 +1214,46 @@ remmina_plugin_vnc_main (RemminaProtocolWidget *gp)
             }
             else
             {
-                cl->listenPort = remminafile->listenport;
+                cl->listenPort = remmina_plugin_service->file_get_int (remminafile, "listenport", 5500);
             }
 
             remmina_plugin_vnc_incoming_connection (gp, cl);
         }
         else
         {
-            remmina_plugin_service->get_server_port (host, 5900, &cl->serverHost, &cl->serverPort);
-            g_free (host);
+            remmina_plugin_service->get_server_port (host, 5900, &s, &cl->serverPort);
+            cl->serverHost = strdup (s);
+            g_free (s);
 
             /* Support short-form (:0, :1) */
             if (cl->serverPort < 100) cl->serverPort += 5900;
         }
+        g_free (host);
+        host = NULL;
 
-        if (remminafile->proxy && remminafile->proxy[0])
+        if (remmina_plugin_service->file_get_string (remminafile, "proxy"))
         {
-            remmina_plugin_service->get_server_port (remminafile->proxy, 5900, &cl->destHost, &cl->destPort);
+#ifdef LIBVNCSERVER_WITH_CLIENT_TLS
+            cl->destHost = cl->serverHost;
+            cl->destPort = cl->serverPort;
+            remmina_plugin_service->get_server_port (remmina_plugin_service->file_get_string (remminafile, "proxy"), 5900,
+                &s, &cl->serverPort);
+            cl->serverHost = strdup (s);
+            g_free (s);
+#endif
         }
 
-        cl->appData.useRemoteCursor = (remminafile->showcursor ? FALSE : TRUE);
+        cl->appData.useRemoteCursor = (remmina_plugin_service->file_get_int (remminafile, "showcursor", FALSE) ? FALSE : TRUE);
 
-        remmina_plugin_vnc_update_quality (cl, remminafile->quality);
-        remmina_plugin_vnc_update_colordepth (cl, remminafile->colordepth);
+        remmina_plugin_vnc_update_quality (cl, remmina_plugin_service->file_get_int (remminafile, "quality", 0));
+        remmina_plugin_vnc_update_colordepth (cl, remmina_plugin_service->file_get_int (remminafile, "colordepth", 8));
         SetFormatAndEncodings (cl);
 
-        if (remminafile->disableencryption)
+        if (remmina_plugin_service->file_get_int (remminafile, "disableencryption", FALSE))
         {
+#ifdef LIBVNCSERVER_WITH_CLIENT_TLS
             SetClientAuthSchemes (cl, remmina_plugin_vnc_no_encrypt_auth_types, -1);
+#endif
         }
 
         if (rfbInitClient (cl, NULL, NULL)) break;
@@ -1249,7 +1274,7 @@ remmina_plugin_vnc_main (RemminaProtocolWidget *gp)
         }
 
         /* Otherwise, it's a password error. Try to clear saved password if any */
-        if (remminafile->password) remminafile->password[0] = '\0';
+        remmina_plugin_service->file_set_string (remminafile, "password", NULL);
 
         if (!gpdata->connected) break;
 
@@ -1281,7 +1306,7 @@ remmina_plugin_vnc_main (RemminaProtocolWidget *gp)
 
     remmina_plugin_service->protocol_plugin_emit_signal (gp, "connect");
 
-    if (remminafile->disableserverinput)
+    if (remmina_plugin_service->file_get_int (remminafile, "disableserverinput", FALSE))
     {
         PermitServerInput(cl, 1);
     }
@@ -1321,7 +1346,7 @@ remmina_plugin_vnc_on_motion (GtkWidget *widget, GdkEventMotion *event, RemminaP
     gpdata = (RemminaPluginVncData*) g_object_get_data (G_OBJECT (gp), "plugin-data");
     if (!gpdata->connected || !gpdata->client) return FALSE;
     remminafile = remmina_plugin_service->protocol_plugin_get_file (gp);
-    if (remminafile->viewonly) return FALSE;
+    if (remmina_plugin_service->file_get_int (remminafile, "viewonly", FALSE)) return FALSE;
 
     if (remmina_plugin_service->protocol_plugin_get_scale (gp))
     {
@@ -1349,7 +1374,7 @@ remmina_plugin_vnc_on_button (GtkWidget *widget, GdkEventButton *event, RemminaP
     gpdata = (RemminaPluginVncData*) g_object_get_data (G_OBJECT (gp), "plugin-data");
     if (!gpdata->connected || !gpdata->client) return FALSE;
     remminafile = remmina_plugin_service->protocol_plugin_get_file (gp);
-    if (remminafile->viewonly) return FALSE;
+    if (remmina_plugin_service->file_get_int (remminafile, "viewonly", FALSE)) return FALSE;
 
     /* We only accept 3 buttons */
     if (event->button < 1 || event->button > 3) return FALSE;
@@ -1386,7 +1411,7 @@ remmina_plugin_vnc_on_scroll (GtkWidget *widget, GdkEventScroll *event, RemminaP
     gpdata = (RemminaPluginVncData*) g_object_get_data (G_OBJECT (gp), "plugin-data");
     if (!gpdata->connected || !gpdata->client) return FALSE;
     remminafile = remmina_plugin_service->protocol_plugin_get_file (gp);
-    if (remminafile->viewonly) return FALSE;
+    if (remmina_plugin_service->file_get_int (remminafile, "viewonly", FALSE)) return FALSE;
 
     switch (event->direction)
     {
@@ -1470,9 +1495,10 @@ remmina_plugin_vnc_on_key (GtkWidget *widget, GdkEventKey *event, RemminaProtoco
     gpdata = (RemminaPluginVncData*) g_object_get_data (G_OBJECT (gp), "plugin-data");
     if (!gpdata->connected || !gpdata->client) return FALSE;
     remminafile = remmina_plugin_service->protocol_plugin_get_file (gp);
-    if (remminafile->viewonly) return FALSE;
+    if (remmina_plugin_service->file_get_int (remminafile, "viewonly", FALSE)) return FALSE;
 
-    keyval = remmina_plugin_service->pref_keymap_get_keyval (remminafile->gkeymap, event->keyval);
+    keyval = remmina_plugin_service->pref_keymap_get_keyval (
+        remmina_plugin_service->file_get_string (remminafile, "gkeymap"), event->keyval);
 
     remmina_plugin_vnc_event_push (gp, REMMINA_PLUGIN_VNC_EVENT_KEY, GUINT_TO_POINTER (keyval),
         GINT_TO_POINTER (event->type == GDK_KEY_PRESS ? TRUE : FALSE), NULL);
@@ -1522,7 +1548,7 @@ remmina_plugin_vnc_on_cuttext (GtkClipboard *clipboard, GdkEvent *event, Remmina
     gpdata = (RemminaPluginVncData*) g_object_get_data (G_OBJECT (gp), "plugin-data");
     if (!gpdata->connected || !gpdata->client) return;
     remminafile = remmina_plugin_service->protocol_plugin_get_file (gp);
-    if (remminafile->viewonly) return;
+    if (remmina_plugin_service->file_get_int (remminafile, "viewonly", FALSE)) return;
 
     gtk_clipboard_request_text (clipboard, (GtkClipboardTextReceivedFunc) remmina_plugin_vnc_on_cuttext_request, gp);
 }
@@ -1539,7 +1565,7 @@ remmina_plugin_vnc_on_realize (RemminaProtocolWidget *gp, gpointer data)
     gpdata = (RemminaPluginVncData*) g_object_get_data (G_OBJECT (gp), "plugin-data");
     remminafile = remmina_plugin_service->protocol_plugin_get_file (gp);
 
-    if (remminafile->showcursor)
+    if (remmina_plugin_service->file_get_int (remminafile, "showcursor", FALSE))
     {
         /* Hide local cursor (show a small dot instead) */
         gdk_color_parse ("black", &fg);
@@ -1584,7 +1610,7 @@ remmina_plugin_vnc_open_connection (RemminaProtocolWidget *gp)
     g_signal_connect (G_OBJECT (gpdata->drawing_area), "key-release-event",
         G_CALLBACK (remmina_plugin_vnc_on_key), gp);
 
-    if (!remminafile->disableclipboard)
+    if (!remmina_plugin_service->file_get_int (remminafile, "disableclipboard", FALSE))
     {
         gpdata->clipboard_handler = g_signal_connect (G_OBJECT (gtk_clipboard_get (GDK_SELECTION_CLIPBOARD)),
             "owner-change", G_CALLBACK (remmina_plugin_vnc_on_cuttext), gp);
@@ -1710,66 +1736,60 @@ remmina_plugin_vnc_close_connection (RemminaProtocolWidget *gp)
     return FALSE;
 }
 
-static gpointer
-remmina_plugin_vnc_query_feature (RemminaProtocolWidget *gp, RemminaProtocolFeature feature)
+static gboolean
+remmina_plugin_vnc_query_feature (RemminaProtocolWidget *gp, const RemminaProtocolFeature *feature)
 {
     RemminaPluginVncData *gpdata;
 
     gpdata = (RemminaPluginVncData*) g_object_get_data (G_OBJECT (gp), "plugin-data");
-    switch (feature)
+    switch (feature->id)
     {
-        case REMMINA_PROTOCOL_FEATURE_PREF:
-        case REMMINA_PROTOCOL_FEATURE_PREF_QUALITY:
-        case REMMINA_PROTOCOL_FEATURE_PREF_VIEWONLY:
-        case REMMINA_PROTOCOL_FEATURE_UNFOCUS:
-        case REMMINA_PROTOCOL_FEATURE_SCALE:
-        case REMMINA_PROTOCOL_FEATURE_TOOL_REFRESH:
-            return GINT_TO_POINTER (1);
-        case REMMINA_PROTOCOL_FEATURE_PREF_DISABLESERVERINPUT:
+        case REMMINA_PLUGIN_VNC_FEATURE_PREF_DISABLESERVERINPUT:
             return (SupportsClient2Server ((rfbClient*) (gpdata->client), rfbSetServerInput) ?
-                GINT_TO_POINTER (1) : NULL);
-        case REMMINA_PROTOCOL_FEATURE_TOOL_CHAT:
+                TRUE : FALSE);
+        case REMMINA_PLUGIN_VNC_FEATURE_TOOL_CHAT:
             return (SupportsClient2Server ((rfbClient*) (gpdata->client), rfbTextChat) ?
-                GINT_TO_POINTER (1) : NULL);
+                TRUE : FALSE);
         default:
-            return NULL;
+            return TRUE;
     }
 }
 
 static void
-remmina_plugin_vnc_call_feature (RemminaProtocolWidget *gp, RemminaProtocolFeature feature, const gpointer data)
+remmina_plugin_vnc_call_feature (RemminaProtocolWidget *gp, const RemminaProtocolFeature *feature)
 {
     RemminaPluginVncData *gpdata;
     RemminaFile *remminafile;
 
     gpdata = (RemminaPluginVncData*) g_object_get_data (G_OBJECT (gp), "plugin-data");
     remminafile = remmina_plugin_service->protocol_plugin_get_file (gp);
-    switch (feature)
+    switch (feature->id)
     {
-        case REMMINA_PROTOCOL_FEATURE_PREF_QUALITY:
-            remmina_plugin_vnc_update_quality ((rfbClient*) (gpdata->client), GPOINTER_TO_INT (data));
+        case REMMINA_PLUGIN_VNC_FEATURE_PREF_QUALITY:
+            remmina_plugin_vnc_update_quality ((rfbClient*) (gpdata->client),
+                remmina_plugin_service->file_get_int (remminafile, "quality", 0));
             SetFormatAndEncodings ((rfbClient*) (gpdata->client));
             break;
-        case REMMINA_PROTOCOL_FEATURE_PREF_VIEWONLY:
-            remminafile->viewonly = (data != NULL);
+        case REMMINA_PLUGIN_VNC_FEATURE_PREF_VIEWONLY:
             break;
-        case REMMINA_PROTOCOL_FEATURE_PREF_DISABLESERVERINPUT:
-            PermitServerInput ((rfbClient*) (gpdata->client), (data ? 1 : 0));
-            remminafile->disableserverinput = (data ? TRUE : FALSE);
+        case REMMINA_PLUGIN_VNC_FEATURE_PREF_DISABLESERVERINPUT:
+            PermitServerInput ((rfbClient*) (gpdata->client),
+                remmina_plugin_service->file_get_int (remminafile, "disableserverinput", FALSE) ? 1 : 0);
             break;
-        case REMMINA_PROTOCOL_FEATURE_UNFOCUS:
+        case REMMINA_PLUGIN_VNC_FEATURE_UNFOCUS:
             remmina_plugin_vnc_release_key (gp, 0);
             break;
-        case REMMINA_PROTOCOL_FEATURE_SCALE:
-            remmina_plugin_vnc_update_scale (gp, (data != NULL));
+        case REMMINA_PLUGIN_VNC_FEATURE_SCALE:
+            remmina_plugin_vnc_update_scale (gp,
+                remmina_plugin_service->file_get_int (remminafile, "scale", FALSE));
             break;
-        case REMMINA_PROTOCOL_FEATURE_TOOL_REFRESH:
+        case REMMINA_PLUGIN_VNC_FEATURE_TOOL_REFRESH:
             SendFramebufferUpdateRequest ((rfbClient*) (gpdata->client), 0, 0,
                 remmina_plugin_service->protocol_plugin_get_width (gp),
                 remmina_plugin_service->protocol_plugin_get_height (gp),
                 FALSE);
             break;
-        case REMMINA_PROTOCOL_FEATURE_TOOL_CHAT:
+        case REMMINA_PLUGIN_VNC_FEATURE_TOOL_CHAT:
             remmina_plugin_vnc_open_chat (gp);
             break;
         default:
@@ -1874,64 +1894,93 @@ remmina_plugin_vnc_init (RemminaProtocolWidget *gp)
 #endif
 }
 
+static gpointer colordepth_list[] =
+{
+    "8", N_("256 colors"),
+    "15", N_("High color (15 bit)"),
+    "16", N_("High color (16 bit)"),
+    "24", N_("True color (24 bit)"),
+    NULL
+};
+
+static gpointer quality_list[] =
+{
+    "0", N_("Poor (fastest)"),
+    "1", N_("Medium"),
+    "2", N_("Good"),
+    "9", N_("Best (slowest)"),
+    NULL
+};
+
 static const RemminaProtocolSetting remmina_plugin_vnc_basic_settings[] =
 {
-    REMMINA_PROTOCOL_SETTING_SERVER,
-    REMMINA_PROTOCOL_SETTING_PROXY_DEST,
-    REMMINA_PROTOCOL_SETTING_USERNAME,
-    REMMINA_PROTOCOL_SETTING_PASSWORD,
-    REMMINA_PROTOCOL_SETTING_COLORDEPTH,
-    REMMINA_PROTOCOL_SETTING_QUALITY,
-    REMMINA_PROTOCOL_SETTING_CTL_CONCAT,
-    REMMINA_PROTOCOL_SETTING_SHOWCURSOR_REMOTE,
-    REMMINA_PROTOCOL_SETTING_VIEWONLY,
-    REMMINA_PROTOCOL_SETTING_CTL_CONCAT_END,
-    REMMINA_PROTOCOL_SETTING_CTL_CONCAT,
-    REMMINA_PROTOCOL_SETTING_DISABLECLIPBOARD,
-    REMMINA_PROTOCOL_SETTING_DISABLEENCRYPTION,
-    REMMINA_PROTOCOL_SETTING_CTL_CONCAT_END,
-    REMMINA_PROTOCOL_SETTING_DISABLESERVERINPUT,
-    REMMINA_PROTOCOL_SETTING_CTL_END
+    { REMMINA_PROTOCOL_SETTING_TYPE_SERVER, NULL, NULL, FALSE, "_rfb._tcp", NULL },
+#ifdef LIBVNCSERVER_WITH_CLIENT_TLS
+    { REMMINA_PROTOCOL_SETTING_TYPE_TEXT, "proxy", N_("Repeater"), FALSE, NULL, NULL },
+#endif
+    { REMMINA_PROTOCOL_SETTING_TYPE_TEXT, "username", N_("User name"), FALSE, NULL, NULL },
+    { REMMINA_PROTOCOL_SETTING_TYPE_PASSWORD, NULL, NULL, FALSE, NULL, NULL },
+    { REMMINA_PROTOCOL_SETTING_TYPE_SELECT, "colordepth", N_("Color depth"), FALSE, colordepth_list, NULL },
+    { REMMINA_PROTOCOL_SETTING_TYPE_SELECT, "quality", N_("Quality"), FALSE, quality_list, NULL },
+    { REMMINA_PROTOCOL_SETTING_TYPE_CHECK, "showcursor", N_("Show remote cursor"), TRUE, NULL, NULL },
+    { REMMINA_PROTOCOL_SETTING_TYPE_CHECK, "viewonly", N_("View only"), FALSE, NULL, NULL },
+    { REMMINA_PROTOCOL_SETTING_TYPE_CHECK, "disableclipboard", N_("Disable clipboard sync"), TRUE, NULL, NULL },
+    { REMMINA_PROTOCOL_SETTING_TYPE_CHECK, "disableencryption", N_("Disable encryption"), FALSE, NULL, NULL },
+    { REMMINA_PROTOCOL_SETTING_TYPE_CHECK, "disableserverinput", N_("Disable server input"), FALSE, NULL, NULL },
+    { REMMINA_PROTOCOL_SETTING_TYPE_END, NULL, NULL, FALSE, NULL, NULL }
 };
 
 static const RemminaProtocolSetting remmina_plugin_vnci_basic_settings[] =
 {
-    REMMINA_PROTOCOL_SETTING_LISTENPORT,
-    REMMINA_PROTOCOL_SETTING_USERNAME,
-    REMMINA_PROTOCOL_SETTING_PASSWORD,
-    REMMINA_PROTOCOL_SETTING_COLORDEPTH,
-    REMMINA_PROTOCOL_SETTING_QUALITY,
-    REMMINA_PROTOCOL_SETTING_CTL_CONCAT,
-    REMMINA_PROTOCOL_SETTING_SHOWCURSOR_REMOTE,
-    REMMINA_PROTOCOL_SETTING_VIEWONLY,
-    REMMINA_PROTOCOL_SETTING_CTL_CONCAT_END,
-    REMMINA_PROTOCOL_SETTING_CTL_CONCAT,
-    REMMINA_PROTOCOL_SETTING_DISABLECLIPBOARD,
-    REMMINA_PROTOCOL_SETTING_DISABLEENCRYPTION,
-    REMMINA_PROTOCOL_SETTING_CTL_CONCAT_END,
-    REMMINA_PROTOCOL_SETTING_DISABLESERVERINPUT,
-    REMMINA_PROTOCOL_SETTING_CTL_END
+    { REMMINA_PROTOCOL_SETTING_TYPE_TEXT, "listenport", N_("Listen on port"), FALSE, NULL, NULL },
+    { REMMINA_PROTOCOL_SETTING_TYPE_TEXT, "username", N_("User name"), FALSE, NULL, NULL },
+    { REMMINA_PROTOCOL_SETTING_TYPE_PASSWORD, NULL, NULL, FALSE, NULL, NULL },
+    { REMMINA_PROTOCOL_SETTING_TYPE_SELECT, "colordepth", N_("Color depth"), FALSE, colordepth_list, NULL },
+    { REMMINA_PROTOCOL_SETTING_TYPE_SELECT, "quality", N_("Quality"), FALSE, quality_list, NULL },
+    { REMMINA_PROTOCOL_SETTING_TYPE_CHECK, "showcursor", N_("Show remote cursor"), TRUE, NULL, NULL },
+    { REMMINA_PROTOCOL_SETTING_TYPE_CHECK, "viewonly", N_("View only"), FALSE, NULL, NULL },
+    { REMMINA_PROTOCOL_SETTING_TYPE_CHECK, "disableclipboard", N_("Disable clipboard sync"), TRUE, NULL, NULL },
+    { REMMINA_PROTOCOL_SETTING_TYPE_CHECK, "disableencryption", N_("Disable encryption"), FALSE, NULL, NULL },
+    { REMMINA_PROTOCOL_SETTING_TYPE_CHECK, "disableserverinput", N_("Disable server input"), FALSE, NULL, NULL },
+    { REMMINA_PROTOCOL_SETTING_TYPE_END, NULL, NULL, FALSE, NULL, NULL }
 };
 
 static const RemminaProtocolSetting remmina_plugin_vnc_advanced_settings[] =
 {
-    REMMINA_PROTOCOL_SETTING_SCALE,
-    REMMINA_PROTOCOL_SETTING_GKEYMAP,
-    REMMINA_PROTOCOL_SETTING_CTL_END
+    { REMMINA_PROTOCOL_SETTING_TYPE_SCALE, NULL, NULL, FALSE, NULL, NULL },
+    { REMMINA_PROTOCOL_SETTING_TYPE_KEYMAP, NULL, NULL, FALSE, NULL, NULL },
+    { REMMINA_PROTOCOL_SETTING_TYPE_END, NULL, NULL, FALSE, NULL, NULL }
+};
+
+static const RemminaProtocolFeature remmina_plugin_vnc_features[] =
+{
+    { REMMINA_PROTOCOL_FEATURE_TYPE_PREF, REMMINA_PLUGIN_VNC_FEATURE_PREF_QUALITY,
+      GINT_TO_POINTER (REMMINA_PROTOCOL_FEATURE_PREF_RADIO), "quality", quality_list },
+    { REMMINA_PROTOCOL_FEATURE_TYPE_PREF, REMMINA_PLUGIN_VNC_FEATURE_PREF_VIEWONLY,
+      GINT_TO_POINTER (REMMINA_PROTOCOL_FEATURE_PREF_CHECK), "viewonly", N_("View only") },
+    { REMMINA_PROTOCOL_FEATURE_TYPE_PREF, REMMINA_PLUGIN_VNC_FEATURE_PREF_DISABLESERVERINPUT,
+      GINT_TO_POINTER (REMMINA_PROTOCOL_FEATURE_PREF_CHECK), "disableserverinput", N_("Disable server input") },
+    { REMMINA_PROTOCOL_FEATURE_TYPE_TOOL, REMMINA_PLUGIN_VNC_FEATURE_TOOL_REFRESH, N_("Refresh"), GTK_STOCK_REFRESH, NULL },
+    { REMMINA_PROTOCOL_FEATURE_TYPE_TOOL, REMMINA_PLUGIN_VNC_FEATURE_TOOL_CHAT, N_("Open Chat..."), "face-smile", NULL },
+    { REMMINA_PROTOCOL_FEATURE_TYPE_SCALE, REMMINA_PLUGIN_VNC_FEATURE_SCALE, NULL, NULL, NULL },
+    { REMMINA_PROTOCOL_FEATURE_TYPE_UNFOCUS, REMMINA_PLUGIN_VNC_FEATURE_UNFOCUS, NULL, NULL, NULL },
+    { REMMINA_PROTOCOL_FEATURE_TYPE_END, 0, NULL, NULL, NULL }
 };
 
 static RemminaProtocolPlugin remmina_plugin_vnc =
 {
     REMMINA_PLUGIN_TYPE_PROTOCOL,
     "VNC",
-    NULL,
+    N_("VNC - Virtual Network Computing"),
+    GETTEXT_PACKAGE,
+    VERSION,
 
     "remmina-vnc",
     "remmina-vnc-ssh",
-    "_rfb._tcp",
     remmina_plugin_vnc_basic_settings,
     remmina_plugin_vnc_advanced_settings,
     REMMINA_PROTOCOL_SSH_SETTING_TUNNEL,
+    remmina_plugin_vnc_features,
 
     remmina_plugin_vnc_init,
     remmina_plugin_vnc_open_connection,
@@ -1944,14 +1993,16 @@ static RemminaProtocolPlugin remmina_plugin_vnci =
 {
     REMMINA_PLUGIN_TYPE_PROTOCOL,
     "VNCI",
-    NULL,
+    N_("VNC - Incoming Connection"),
+    GETTEXT_PACKAGE,
+    VERSION,
 
     "remmina-vnc",
     "remmina-vnc-ssh",
-    NULL,
     remmina_plugin_vnci_basic_settings,
     remmina_plugin_vnc_advanced_settings,
     REMMINA_PROTOCOL_SSH_SETTING_REVERSE_TUNNEL,
+    remmina_plugin_vnc_features,
 
     remmina_plugin_vnc_init,
     remmina_plugin_vnc_open_connection,
@@ -1968,13 +2019,11 @@ remmina_plugin_entry (RemminaPluginService *service)
     bindtextdomain (GETTEXT_PACKAGE, REMMINA_LOCALEDIR);
     bind_textdomain_codeset (GETTEXT_PACKAGE, "UTF-8");
 
-    remmina_plugin_vnc.description = _("VNC - Virtual Network Computing");
     if (! service->register_plugin ((RemminaPlugin *) &remmina_plugin_vnc))
     {
         return FALSE;
     }
 
-    remmina_plugin_vnci.description = _("VNC - Incoming Connection");
     if (! service->register_plugin ((RemminaPlugin *) &remmina_plugin_vnci))
     {
         return FALSE;
